#!/usr/local/bin/perl -w

#### LAST UPDATED: Oct19, 2002 ####
# pretag.pl version 0.1

###########################################################################
#
# This program pre-tags the head words in the senseval data with its part
# of speech. The general class as in noun, verb etc... is obtained from 
# the senseval data itself; the exact part of speech is assigned by 
# manual tagging based on the morphological form of the token. Exceptions
# such as capitalised head words at begining of sentence are handled
# by using the -PNouns option and a file which holds information for
# specific instances.
#
# USAGE : pretag.pl [OPTIONS] TAGS DESTINATION SOURCE [SOURCE]... 
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("PNouns=s", "version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
if ( defined $opt_PNouns )
{
    $enablepnouns = 1;
    $Pnounfl = $opt_PNouns;
}
else { $enablepnouns = 0; }

# Print help if asked
if ( $help )
{
    &help();
    exit;
}
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 2 )
{
    &usage();
    exit;
}

# Accept command line arguement - TAG file name
$tagfl = shift(@ARGV);
# Open the pre TAG file
open(PTG,"<$tagfl") or die "Could not open the pretags file\n";
# If PNouns option used: open SPTAG file
if($enablepnouns == 1)
{
    open(NN,"<$Pnounfl") or die "Could not open the PNoun file\n";
}
# Accept destination file name
$opfl = shift(@ARGV);

if(-e $opfl)
{
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y")
    {
        print "Overwriting the file.\n";
    }
    else
    {
        exit;
    }
}

# Open destination file
open(OP,">$opfl") or die "Could not open output file\n";

# Read lines from the source file(s) line by line
while($eng = <>)
{
    # Increment line number
    $lnno++;

    # CHANGE
    # Capture broad class of the task from lexelt tag
    if($eng =~ /^\s*<lexelt item=\".*([a-z])\">/) {
	$bclass = $1;
	if($bclass eq "n") 	{ $btag = "N"; }
	elsif($bclass eq "a")   { $btag = "J"; }
	elsif($bclass eq "v")   { $btag = "V"; }
	else			{ $btag = "P"; }
    }


    # Check if line has head tag
    # if($eng =~ /<head[^>]*>\s*([^>\s]+)\s*</)
    if($eng =~ /<head[^>]*>\s*([^>]+)\s*</)
    {
	# CHANGE FEB16
	$allhds = $1;
	$allhds =~ s/\s*$//;
	$allhds =~ s/'/ '/g;
	@hdwds = split /\s/, $allhds;
	$ptghdwds ="";
	foreach $hdwd (@hdwds) {
	
	# Capture head(target) word
	# $hdwd = $1;
	# END CHANGE

	# Read lines from the TAG file
	while($ptg = <PTG>)
	{
	    # Read next line in case present line is blank
	    if($ptg =~ /^\s*$/)
	    { next; }
	    # Check if line has info for present head word
	    # CHANGE
	    # if($ptg =~ /^$hdwd ([A-Z]+)/)
	    if($ptg =~ /^$hdwd (($btag)[A-Z]+)/)
	    {
		$morphtag = $1;
		$hdwd =~ s/$hdwd/$hdwd\/\/$morphtag/;
		# Capture the preassigned tag
		# CHANGE FEB17
		# $ptaghd = $1;	
		# $hdwd =~ s/$hdwd/$hdwd\/\/$1/;			
		# Pretagging
		# $eng =~ s/(<head[^>]*>\s*[^>\s]+)(\s*<)/$1\/\/$ptaghd$2/;
		last;
	    }
	}
	# Reset file handle to point to begining of file
	seek PTG, 0, 0;
	
	# Check if PNouns option specified
	if($enablepnouns==1)
	{
	    # Read lines from the SPTAG file
	    while($nn = <NN>)
	    {
		# Read next line in case present line is blank
		if($nn =~ /^\s*$/)
		{ next; }
		# Check if line has info for present head word (based on line no.)
		if($nn =~ /\s$lnno\s/)
		{
		    # Default tag is common noun
		    $ptaghd = "NN";
		    # Checkk if tag specified
		    if($nn =~ /\s$lnno\s+([A-Z]+)/)
		    {
			# Capture the tag
			$ptaghd = $1;
			# CHANGE FEB17
			$hdwd =~ s/\/\/.*/\/\/$ptaghd/;
		    }
		    # Tag the instance	
		    # CHANGE FEB17
		    # $eng =~ s/(<head[^>]*>\s*[^>\s\/]+)\s*[^\s<]+(\s*<)/$1\/\/$ptaghd$2/;
		    
		    last;
		}
	    }
	    # Reset file handle to point to begining of file
	    seek NN, 0, 0;
	}

	# CHANGE FEB16
	if($ptghdwds eq "") 	{ $ptghdwds = $hdwd; }
	else			{ $ptghdwds = $ptghdwds." ".$hdwd; }
	}
	$eng =~ s/(<head[^>]*>)\s*([^<]+)</$1$ptghdwds</;
	# END CHANGE
    }
    # Print line to DESTINATION file
    print OP $eng;
}

# Subroutine to print help message
sub help
{
    print "Usage: pretag.pl [OPTIONS] TAGS DESTINATION SOURCE [SOURCE]... \n\n";
    print "This program pre-tags the head words in the SOURCE file(s) with its part of speech as stored in TAGS; the final output being the DESTINATION file. The general class as in noun, verb etc... is obtained from the senseval data itself; the exact part of speech is assigned by manual tagging based on the morphological form of the token. Exceptions such as capitalised head words at begining of sentence are handled by using the -PNouns option.\n\n"; 
    print "OPTIONS:\n\n";
    print "  --help             Prints the help message.\n";
    print "  --PNouns PNOUNS    Used to tag specific instances of the head word.\n";
    print "                     Instance information stored in file PNOUNS.\n";
    print "  --version          Prints the version number.\n\n";
}

sub usage
{
    print "Usage: pretag.pl [OPTIONS] TAGS DESTINATION SOURCE [SOURCE]... \n";
    AskHelp();
}
# Message describing way to get help
sub AskHelp
{
    print "Type pretag.pl --help for help.\n";
}
# function to output the version number
sub showVersion
{
    print STDERR "pretag.pl      -        version 0.1\n";
    print STDERR "A component of refine, version 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

