#!/usr/local/bin/perl -w

#### LAST UPDATED: Dec, 2003 ####
# unicode2xml.pl version 0.1

###########################################################################
#
# This program replaces special unicode characters as listed in the file
# UNIMAP by appropriate xml tags which are also listed in UNIMAP.
# UNIMAP is to have one entry per unicode character. Each entry is on
# one line and has the unicode character, white space and the corresponding
# xml representation, in that order.
#
# USAGE :  unicode2xml.pl [OPTIONS] UNIMAP DESTINATION SOURCE
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}


# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 2 ) {
    &usage();
    exit;}

$unimapfl = shift();
$opfl = shift();
$inpfl = shift();

if(-f $opfl) {  
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
	print "Overwriting the file.\n"; }
    else{
	exit; } }
 
if(!(-f $unimapfl)){ 
    print "UNIMAP file does not exist.\n";
    exit; }  
if(!(-f $inpfl)){
    print "SOURCE file does not exist.\n";
    exit; }

# Open the various files
open(INP,"<$inpfl");
open(UNI,"<$unimapfl");
open(OP,">$opfl");

# Store Unicode mapping into a hash
while($ln = <UNI>) {
	if($ln =~ /^\s*([^\s]+)\s+([^\s]+)/) {
		$unihash{$1} = $2;
	}
}

$incontext = 0;
# Read lines from input file
while($ln = <INP>) {
    # Set variable based on whether line part of context or not
    if($ln =~ /^\s*<context>\s*$/) {
	$incontext = 1;
	print OP $ln;
	next; }
    if($ln =~ /^\s*<\/context>\s*$/) {
	$incontext = 0; 
	print OP $ln;
	next; }
    
    # Process only if line part of context
    if($incontext){
	# Ignore blank lines
	if($ln =~ /^\s*$/){
	    next;}

	# @arr=sort{($unihash{$a}) <=> ($unihash{$b}) } keys %unihash;
	foreach $element (keys %unihash)
        {
		$ln =~ s/$element/ $unihash{$element} /g;
        }
    }

    print OP $ln; 
}

# Subroutine to print help message
sub help{
    print "Usage: unicode2xml.pl [OPTIONS] UNIMAP DESTINATION SOURCE\n\n"; 
    print "This program replaces special unicode characters as listed in the file UNIMAP by appropriate xml tags which are also listed in UNIMAP. UNIMAP is to have one entry per unicode character. Each entry is on one line and has the unicode character, white space and the corresponding xml representation, in that order.\n\n";
    print "OPTIONS:\n";
    print "  --help             Prints the help message.\n";
    print "  --version          Prints the version number.\n\n"; }

sub usage{
    print "Usage: unicode2xml.pl [OPTIONS] UNIMAP DESTINATION SOURCE\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type unicode2xml.pl --help for help.\n";}
# function to output the version number
sub showVersion
{
    print STDERR "unicode2xml.pl      -        version 0.1\n";
    print STDERR "A component of refine version 0.2\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

